
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "types.h"
/* enumerations from "/usr/src/ports/gnome-recipes/gnome-recipes-2.0.2-1.x86_64/src/gnome-recipes-2.0.2/x86_64-pc-cygwin/../src/gr-diet.h" */
GType
gr_diets_get_type (void)
{
    static GType etype = 0;
    if (G_UNLIKELY(etype == 0)) {
        static const GFlagsValue values[] = {
            { GR_DIET_GLUTEN_FREE, "GR_DIET_GLUTEN_FREE", "gluten-free" },
            { GR_DIET_NUT_FREE, "GR_DIET_NUT_FREE", "nut-free" },
            { GR_DIET_VEGAN, "GR_DIET_VEGAN", "vegan" },
            { GR_DIET_VEGETARIAN, "GR_DIET_VEGETARIAN", "vegetarian" },
            { GR_DIET_MILK_FREE, "GR_DIET_MILK_FREE", "milk-free" },
            { 0, NULL, NULL }
        };
        etype = g_flags_register_static (g_intern_static_string ("GrDiets"), values);
    }
    return etype;
}
/* enumerations from "/usr/src/ports/gnome-recipes/gnome-recipes-2.0.2-1.x86_64/src/gnome-recipes-2.0.2/x86_64-pc-cygwin/../src/gr-unit.h" */
GType
gr_unit_get_type (void)
{
    static GType etype = 0;
    if (G_UNLIKELY(etype == 0)) {
        static const GEnumValue values[] = {
            { GR_UNIT_UNKNOWN, "GR_UNIT_UNKNOWN", "unit-unknown" },
            { GR_UNIT_NONE, "GR_UNIT_NONE", "unit-none" },
            { GR_UNIT_NUMBER, "GR_UNIT_NUMBER", "unit-number" },
            { GR_UNIT_GRAM, "GR_UNIT_GRAM", "unit-gram" },
            { GR_UNIT_KILOGRAM, "GR_UNIT_KILOGRAM", "unit-kilogram" },
            { GR_UNIT_POUND, "GR_UNIT_POUND", "unit-pound" },
            { GR_UNIT_OUNCE, "GR_UNIT_OUNCE", "unit-ounce" },
            { GR_UNIT_LITER, "GR_UNIT_LITER", "unit-liter" },
            { GR_UNIT_DECILITER, "GR_UNIT_DECILITER", "unit-deciliter" },
            { GR_UNIT_MILLILITER, "GR_UNIT_MILLILITER", "unit-milliliter" },
            { GR_UNIT_FLUID_OUNCE, "GR_UNIT_FLUID_OUNCE", "unit-fluid-ounce" },
            { GR_UNIT_PINT, "GR_UNIT_PINT", "unit-pint" },
            { GR_UNIT_QUART, "GR_UNIT_QUART", "unit-quart" },
            { GR_UNIT_GALLON, "GR_UNIT_GALLON", "unit-gallon" },
            { GR_UNIT_CUP, "GR_UNIT_CUP", "unit-cup" },
            { GR_UNIT_TABLESPOON, "GR_UNIT_TABLESPOON", "unit-tablespoon" },
            { GR_UNIT_TEASPOON, "GR_UNIT_TEASPOON", "unit-teaspoon" },
            { GR_UNIT_BOX, "GR_UNIT_BOX", "unit-box" },
            { GR_UNIT_PACKAGE, "GR_UNIT_PACKAGE", "unit-package" },
            { GR_UNIT_GLASS, "GR_UNIT_GLASS", "unit-glass" },
            { GR_UNIT_MILLIMETER, "GR_UNIT_MILLIMETER", "unit-millimeter" },
            { GR_UNIT_CENTIMETER, "GR_UNIT_CENTIMETER", "unit-centimeter" },
            { GR_UNIT_METER, "GR_UNIT_METER", "unit-meter" },
            { GR_UNIT_STONE, "GR_UNIT_STONE", "unit-stone" },
            { GR_UNIT_PINCH, "GR_UNIT_PINCH", "unit-pinch" },
            { GR_UNIT_BUNCH, "GR_UNIT_BUNCH", "unit-bunch" },
            { GR_LAST_UNIT, "GR_LAST_UNIT", "last-unit" },
            { 0, NULL, NULL }
        };
        etype = g_enum_register_static (g_intern_static_string ("GrUnit"), values);
    }
    return etype;
}
GType
gr_dimension_get_type (void)
{
    static GType etype = 0;
    if (G_UNLIKELY(etype == 0)) {
        static const GEnumValue values[] = {
            { GR_DIMENSION_NONE, "GR_DIMENSION_NONE", "none" },
            { GR_DIMENSION_DISCRETE, "GR_DIMENSION_DISCRETE", "discrete" },
            { GR_DIMENSION_VOLUME, "GR_DIMENSION_VOLUME", "volume" },
            { GR_DIMENSION_MASS, "GR_DIMENSION_MASS", "mass" },
            { GR_DIMENSION_LENGTH, "GR_DIMENSION_LENGTH", "length" },
            { 0, NULL, NULL }
        };
        etype = g_enum_register_static (g_intern_static_string ("GrDimension"), values);
    }
    return etype;
}


/* Generated data ends here */

